<?php
/*********************************************************
 * @version    SVN: $Id$
 * @author     Rich Raffals
 ***********************************************************
 */
require('Form.php');

class DataEntryForm extends Form {
  private $aDbColumnNames = array('word', 'count', 'url');
  public $aWordCount = array(); // key: word, value: count (for $this->url)
  public $url = '';
  public $totalWords = 0;

  ///////////////////////////////////////////////////////////////////
  public function __construct( $queryParam=NULL ) {
  ///////////////////////////////////////////////////////////////////
    parent::__construct($queryParam);
  }

  //
  // Methods
  //
  ///////////////////////////////////////////////////////////////////
  public function sanitize( $sParamName='URL' ) {
  ///////////////////////////////////////////////////////////////////
  // Set $this->url to safe/validated/sanitized/standardized version of $sParamName and return it
  // Calls $this->reportError() if can't produce a safe url from $sParamName

    ///////////////////////////////////////////////////////////////////
    function standardizeUrl($origUrl=NULL) {
    ///////////////////////////////////////////////////////////////////
    // Collapse "http://blah.com","http://blah.com/", "http://Blah.com", and
    //  "http://blah.com/#anchor" TO==> "http://www.blah.com"
    // NOTE: Expects $origUrl to start with '<protocol>://' (eg, 'http://')

      if ( ! empty($origUrl) ) {
        // Remove everything after first '#'
        $aStandardUrl = explode('#', $origUrl);
        if ( count($aStandardUrl) >= 1 ) { // there's at least one #
          $standardUrl = $aStandardUrl[0];
        }
      } else { // should never get here
        $this->reportError('Empty URL');
      }

      // Remove '/' from end
      $standardUrl = rtrim($standardUrl, '/');

      // Convert domain to lower case and make sure we it starts with 'www.'
      $aStandardUrl = explode('/', $standardUrl);
      $domainName = strtolower($aStandardUrl[2]);
      $aDomainName = explode('.', $domainName);
      if ( RESTRICT_TO_HTTP_ONLY && ($aDomainName[0] !== 'www') ) {
        $aStandardUrl[2] = 'www.' . $domainName;
      } else {
        $aStandardUrl[2] = $domainName;
      }
      $standardUrl = implode('/', $aStandardUrl);

      return $standardUrl;
    } // standardizeUrl()
    ///////////////////////////////////////////////////////////////////

    if ( ! empty($this->sRawQueryParam) ) {
      $safeUrl = parent::sanitize($sParamName); // generic form field sanitization

      // Now, do URL-specific stuff...

      // Add 'http://' if user didn't
      if ( RESTRICT_TO_HTTP_ONLY &&
           (count(explode('http://', $safeUrl)) <= 1) &&
           (count(explode('https://', $safeUrl)) <= 1) ) {
        $safeUrl = 'http://' . $safeUrl;
      }

      if ( ! filter_var($safeUrl, FILTER_VALIDATE_URL) ) {
        $this->reportError('Invalid URL');
      }

      // Standardize URL (ie, collapse "http://blah.com","http://blah.com/",
      // "http://Blah.com", and "http://blah.com/#anchor" TO==> "http://www.blah.com")
      // ...so that we get just one entry in the DB.
      $safeUrl = standardizeUrl($safeUrl);

      // Sanitize URL...
      $safeUrl = filter_var($safeUrl, FILTER_SANITIZE_URL);
      $safeUrl = filter_var($safeUrl, FILTER_SANITIZE_SPECIAL_CHARS, FILTER_FLAG_STRIP_LOW);
      $safeUrl = filter_var($safeUrl, FILTER_SANITIZE_SPECIAL_CHARS, FILTER_FLAG_STRIP_HIGH);
      $safeUrl = str_replace('(', '&#40;', $safeUrl);
      $safeUrl = str_replace(')', '&#41;', $safeUrl);

      return $this->url = $safeUrl;
    } // if (! empty($this->sRawQueryParam))

    // Should never get here...
    $this->reportError('No URL entered');
  } // sanitize()


  ///////////////////////////////////////////////////////////////////
  public function countWordsOnPage() {
  ///////////////////////////////////////////////////////////////////
  // RETURNS: $aWordCount[word, count] and saves as $this->aWordCount
  // Calls $this->reportError() if can't produce a list of words from the webpage
    if ( ! empty($this->url) ) {

      // Attempt to read webpage into string
	  // file_get_contents() no longer works, since server now disallows this (presumably for security)
	  // with 4/4/12 PHP upgrade.  So, use curl insteead.
      //$submittedPage = file_get_contents($this->url);

	  // Grab url into string via curl
	  $c = curl_init();
	  curl_setopt($c, CURLOPT_URL, $this->url);
	  curl_setopt($c, CURLOPT_RETURNTRANSFER, 1); // Return as string instead of outputting
	  curl_setopt($c, CURLOPT_FRESH_CONNECT, 1); // Make sure we get the latest (not cached) version
	  curl_setopt($c, CURLOPT_FOLLOWLOCATION, 1); // Follow "Location:" headers
	  curl_setopt($c, CURLOPT_MAXREDIRS, 5); // Avoid hanging on circular references
	  curl_setopt($c, CURLOPT_HEADER, 0); // Don't include header
	  $submittedPage = curl_exec($c);
	  curl_close($c);

      if ( ($submittedPage === FALSE) || empty($submittedPage) ) {
        $this->reportError('Unable to open web page or find any words on it: <strong>' . $this->url . '</strong>');
      }
    
      if ( ! preg_match('/<body.+<\/body>/si', $submittedPage, $aMatches) ) { // Just parse what's between <body>...</body>
        if ( ! preg_match('/<body.+/si', $submittedPage, $aMatches) ) { // Or, parse <body... if there's no </body> tag
          // Otherwise, with no <body> tag at all, bail
          $this->reportError('Webpage specified has no &lt;body&gt; tag.&nbsp; This means the page either has no content, or all content is in frames.&nbsp; <em>[Note: Since this is a quick &amp; dirty demo application done solely to demonstrate coding proficiency, and I did not think adding the capability to handle frames would demonstrate any further programming skill, I did not take the time to implement frame parsing.]</em>: <strong>' . $this->url . '</strong>');
        }
      }

      // Handle case where 2 words are separated only by HTML tags (and get concatenated later by strip_tags() ) = add a space
      $mainContent = preg_replace('/<(.*?)>/s', '<${1}> ', $aMatches[0]);

      $mainContent = preg_replace('/<script.*?<\/script>/s', ' ', $mainContent); // Remove javascript
      $mainContent = preg_replace('/<style.*?<\/style>/s', ' ', $mainContent); // Remove CSS
      $mainContent = strip_tags($mainContent); // Strip out html tags

      // Convert all HTML entities to their applicable chars, convert all to lowercase
      // NOTE: Remove strtolower() call if want to do case sensitive word matching
      $mainContent = strtolower( html_entity_decode($mainContent) );

      // TODO: Handle contractions as one word (probably via "| PREG_SPLIT_DELIM_CAPTURE")
      $aWords = preg_split("/[\W]+/", $mainContent, NULL, PREG_SPLIT_NO_EMPTY); // Split into words

      // Create hash with words as keys and num of occurances as values
      $aWordCount = array();
      foreach ( $aWords as $word ) {
        if ( strlen($word) > MIN_CHARS_FOR_WORDS_TO_COUNT ) { // restrict to words > MIN_CHARS_FOR_WORDS_TO_COUNT chars
          if ( isset($aWordCount[$word]) ) {
            $aWordCount[$word]++;
          } else {
            $aWordCount[$word] = 1;
          }
        }
      }
      if ( ! $this->totalWords = count($aWordCount) ) {
        $this->reportError("No words found on page: <strong>$this->url</strong><br />-- Did you forget the '.com', '.edu', '.org', etc suffix?");
      }

      ksort( $aWordCount, SORT_LOCALE_STRING ); // sort alphabetically by keys
      $this->aWordCount = $aWordCount;
      $this->bStatus = TRUE;

      return $aWordCount;
    } // if (! empty($this->url))

    // Should never get here...
    $this->reportError("URL is empty");
  } // countWordsOnPage()


  ///////////////////////////////////////////////////////////////////
  public function putWordCountArray2db() {
  ///////////////////////////////////////////////////////////////////
  // Writes DB_WORD_TABLE with records of the form specified in
  //  $this->aDbColumnNames (ie [word, count, url]) using mysql prepared statements
  //  that are SQL-injection-safe; Removes any previous records for the current url
  // Calls $this->reportError() on application failure; Throws exception on DB errors
    require(PHP_REQUIRE_ROOT . 'Model/MySQLiWrapper.php');

    try { // connect to MySQL
      $db = MySQLiWrapper::getInstance( array(DB_HOST, DB_USER, DB_PSWD, DB_NAME, DB_PORT) );
    } catch(Exception $e) {
      $this->reportError("Can't access DB: " . $e->getMessage());
    }

    // If we already have rows in words table for $this->url, delete them
    $query = 'DELETE FROM ' .DB_WORD_TABLE. ' WHERE url=?';
    $bindParam = $this->url;  $sBindType = 's';
    try {
      $stmt =  $db->executePrepared($query, $sBindType, $bindParam);
    } catch(Exception $e) {
      $this->reportError('Unable to retrieve info from DB: ' . $e->getMessage());
    }

    // create/execute query to insert words into db
    $query = 'INSERT INTO ' .DB_WORD_TABLE. ' (' .implode(', ', $this->aDbColumnNames). ') VALUES (?, ?, ?)';
    $sBindTypes='sss';
    try {
        if ( ! $stmt->prepare($query) ) {
          throw new Exception("MySQLi prepare: QUERY=prepare($query), ERROR=" . $db->error);
        }

        // Put each word into DB table by executing prepared query bound with proper params
        foreach ($this->aWordCount as $key => $value) {
          if ( $stmt->bind_param($sBindTypes, $key, $value, $this->url) === FALSE) {
            throw new Exception("MySQLi bind_param($sBindTypes, $key, $value, $this->url): " . $db->error);
          }

          if ( ! $stmt->execute() ) { // execute query
            // Attempt to figure out why query failed (maybe a unique field value is already taken?)
            if ($db->errno === 1062 /* ER_DUP_ENTRY */) {
              throw new Exception('This entry already exists in the DB and duplicates are not allowed: ' . $db->error);
            } else {
              throw new Exception("MySQLi execute: QUERY=$query, ERROR=" . $db->error);
            }
          }
        } // foreach
    } catch(Exception $e) {
      $this->reportError('Unable to add info to DB: ' . $e->getMessage());
    }

    $stmt->close();
    $this->bStatus = TRUE;
  } // putWordCountArray2db()

}
?>
