<?php
/*********************************************************
 * @version    SVN: $Id$
 ***********************************************************
 */
 // Gracias to Alejandro Gervasio for his devshed tutorials on this class

class MySQLiWrapper extends MySQLi {

   private static $_instance = NULL;

   // return Singleton instance of MySQLi class (static method)
   ///////////////////////////////////////////////////////////////////
   public static function getInstance(array $config = array()) {
   ///////////////////////////////////////////////////////////////////
     // $config =  array('host', 'user', 'password', 'database', 'port')
     // NOTE: port is optional
       if ( self::$_instance === NULL ) {
          self::$_instance = new self($config);
       }

       return self::$_instance;
   }


   // private constructor (meant to only be called by methods of this class, eg, getInstance() )
   ///////////////////////////////////////////////////////////////////
   private function __construct(array $config) {
   ///////////////////////////////////////////////////////////////////
        if ( count($config) < 4 ) {
            throw new Exception('Invalid number of connection parameters');
        }

        list($host, $user, $password, $database, $port) = $config;
        parent::__construct($host, $user, $password, $database, $port);


        // Can go back to commented ("correct") method if are sure we'll be running on php 5.3 or later...
        // Or, can use this:
        //         if (strnatcmp(phpversion(),'5.3') >= 0) { //Reference is required for PHP 5.3+
        //if ($this->connect_error) {
        /*
         * Use mysqli_connect_error() instead of $connect_error to ensure
         * compatibility with PHP versions prior to 5.2.9 and 5.3.0.
         */
        $sConnectErrorMsg = mysqli_connect_error();
        if ( $sConnectErrorMsg ) {
          //throw new Exception('Error connecting to MySQLi: ' . $this->connect_errno . ' ' . $this->connect_error);
            throw new Exception('Error connecting to MySQLi: ' .  mysqli_connect_error() . ' ' . $sConnectErrorMsg);
        }
   }

   // prevent cloning class instance
   ///////////////////////////////////////////////////////////////////
   private function __clone(){}
   ///////////////////////////////////////////////////////////////////


   // perform query
   ///////////////////////////////////////////////////////////////////
   public function runQuery($query) {
   ///////////////////////////////////////////////////////////////////
     $sMethodName = __METHOD__;
     if (DEBUG_FIREPHP) { global $firephp; $firephp->info($query, "Method $sMethodName() called, with query"); }

     if (is_string($query) AND !empty($query)) {

         if (DEBUG_FIREPHP) { global $firephp; $firephp->info($query, "Executing query"); }
         if ((!$this->real_query($query))) {
           throw new Exception("Error performing QUERY=$query, ERROR=" . $this->error);
         }

         if (DEBUG_FIREPHP) { global $firephp; $firephp->info($this, "Returning MySQLi_ResultWrapper object"); }
         return new MySQLi_ResultWrapper($this);
     }
   }

   ///////////////////////////////////////////////////////////////////
   // get insertion ID
   ///////////////////////////////////////////////////////////////////
   public function getInsertID() {
       return $this->insert_id;
   }

   // close database connection
   ///////////////////////////////////////////////////////////////////
   public function __destruct() {
   ///////////////////////////////////////////////////////////////////
     //$this->close();
     //TODO: Fix this, then UNcomment above
     // Gives this => PHP Warning:  mysqli::close() [<a href='mysqli.close'>mysqli.close</a>]: Couldn't fetch MySQLiWrapper in /Users/rich/web/sharedLib/MySQLiWrapper.class.php on line 82
   }


   // Post PHP 5.3, we need to do use $this->makeValuesReferenced() with call_user_func_array()
   //  [ which is used with $stmt->bind_param(... ]
   ///////////////////////////////////////////////////////////////////
   private static function makeValuesReferenced(&$aOrigArrayOfValues){
   ///////////////////////////////////////////////////////////////////
     if (strnatcmp(phpversion(),'5.3') >= 0) { //Reference is required for PHP 5.3+
	   $aRefs = array();
	   foreach($aOrigArrayOfValues as $key => $value)
		 $aRefs[$key] = &$aOrigArrayOfValues[$key];
	   return $aRefs;
	 }
	 return $aOrigArrayOfValues;
   } // makeValuesReferenced()

   // create/execute prepared statement, return MySQLi_STMT handle for results processing
   // $bindParams can either be a single param, or an array of params
   ///////////////////////////////////////////////////////////////////
   public function executePrepared($query, $sBindTypes, $bindParams) {
   ///////////////////////////////////////////////////////////////////
       $stmt =  $this->stmt_init();
       if ( ! $stmt->prepare($query) ) { 
         throw new Exception("MySQLi prepare: QUERY=prepare($query), ERROR=" . $this->error);
       }

       if ( is_array($bindParams) ) {
           if ( strlen($sBindTypes) != count($bindParams) ) {
             $sBindParams = print_r($bindParams, TRUE);
             throw new Exception("MySQLi bind_param($sBindTypes, $sBindParams): Wrong parameter count, ie # of bind params doesn't match number of bind types");
           }

           // Insert $sBindTypes as 1st elt of $bindParams
           array_unshift( $bindParams, $sBindTypes );




           // $stmt->bind_param(... adjusted for array $bindParams
		   // Prior to PHP 5.3, this was fine:
		   // if ( call_user_func_array( array($stmt, 'bind_param'), $bindParams ) === FALSE) {
		   // Now, tho, we need to send an array of references (not values) to call_user_func_array()
		   if ( call_user_func_array( array($stmt, 'bind_param'), $this->makeValuesReferenced($bindParams) ) === FALSE) {
             $sBindParams = print_r($bindParams, TRUE);
             throw new Exception("MySQLi bind_param($sBindParams): Parameter binding failed; check that # of params bound is same as number of ?'s in query ($query)");
           }
       } else { // we have just one param
           if ( strlen($sBindTypes) != 1 ) {
             throw new Exception("MySQLi bind_param($sBindTypes, $bindParams): Wrong parameter count, ie # of bind params doesn't match number of bind types");
           }

           if ( $stmt->bind_param($sBindTypes, $bindParams) === FALSE ) {
             throw new Exception("MySQLi bind_param($sBindTypes, $bindParams): " . $this->error);
           }
       }

       if ( ! $stmt->execute() ) { // execute query
         // Attempt to figure out why query failed (maybe a unique field value is already taken?)
         if ($this->errno === 1062 /* ER_DUP_ENTRY */) {
           throw new Exception('This entry already exists in the DB and duplicates are not allowed: ' . $this->error);
         } else {
           throw new Exception("MySQLi execute: QUERY=$query, ERROR=" . $this->error);
         }
       }

       return $stmt;
   }

  public function addArray2db($aData, $tbl) {
  // Method that dynamically adds values to MYSQL DB table $tbl using the $aData vars
  // USES: prepared statment protocol for queries that are efficient and SQL-injection safe
  // RETURNS: # of rows added or FALSE if no table cols matched the input array
  //
  // NOTES:
  //   addArray2db() relies on the column names in the DB table matching the names in the HTML form
  //    (ie, the keys in $aData match the column names in $tbl)
  //   addArray2db() expects the DB connection has already been established
  //   Tho prepared statements are used, addArray2db() expects that $aData has been validated/sanitized
  //    by $this->sanitizeRequestVars() or similar (which should also serialize any arrays in $aData)
  //    so that this addArray2db() will automatically handle groups of fields passed as an array, eg
  //    <input type="text" name="FacilNameAry[]" />
  //
  // AJAX HANDLER USAGE:
  //  if (! $db->addArray2db($_REQUEST, FEEDS_TABLE) ) {
  //      $_REQUEST['mysqlError'] = $db->getError();
  //      $_REQUEST['haveError'] = TRUE;
  //  }
  //
  // NORMAL (NON-AJAX HANDLER) USAGE:
  //  if (! $db->addArray2db($_REQUEST, FEEDS_TABLE) ) {
  //      print '<h2> MYSQL ERROR: '.$db->getError().'</h2>'.NL;
  //  } else {
  //      print '<h2> ALL RECORDS WRITTEN TO DB SUCCESSFULLY!</h2>'.NL;
  //  }
  //
    $aTableColumns = array(); // column names as pulled from db table
    $aTableColumns2use = array(); // column names that match $aData form fields
    $aTableValues2use = array(); // parallel array to tableColumns2use, with corresponding values
    $sQuestionMarks = '';  $sBindTypes = '';
 
    // Pull the column names from the table $tbl
    $query = "SHOW COLUMNS FROM ".$tbl;

    if ( ! $pullColumns = $this->runQuery($query) ) {
      throw new Exception("MySQL pull columns: QUERY=$query, ERROR=" . $this->error);
    }

    // Pull an associative array of column names and put them into a non-associative array
    while ( $columns = mysqli_fetch_assoc($pullColumns) ) {
        $aTableColumns[] = $columns["Field"]; // grabs "Field"name field of $pullColumns ary
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($columns['Field'], 'Adding Column from DB to $aTableColumns[]'); }
    }
    if (DEBUG_FIREPHP) { global $firephp; $firephp->info($aTableColumns, '$aTableColumns'); }

    // Check to see if the variables match up with the column names
    // If so, store in $aTableColumns2use
    foreach( $aData as $key => $value ) {
      if ( $value==DEFAULT_NONREQUIRED_FIELD_VALUE ) { $value = DEFAULT_TABLE_INDEX; }
      if (DEBUG_FIREPHP) { global $firephp; $firephp->info($key, '$key'); }
      if (DEBUG_FIREPHP) { global $firephp; $firephp->info($value, '$value'); }

      if ( empty($value) ) { // This is the 1st write of this record, so ignore empty values
          continue; // skip this field entirely
      }

      if ( in_array($key, $aTableColumns) ) {
        // Put the column name into the array
        $aTableColumns2use[] = $key;
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($key, 'Added Column from $aData to $aTableColumns2use[]'); }

        // NOTE: May need to add something like this: $aTableValues2use = addcslashes($value, '%_');
        //       if expect % or _ in data, and will parse later with ...WHERE color LIKE '%red%'
        $aTableValues2use[] = $value;
        $sQuestionMarks .= '?, ';

        // Determine type of each $value for use in the subsequent $stmt->bind_param() call
        if (is_int($value)) {
          $sBindTypes .= 'i'; 
        } else if (is_double($value)) { 
          $sBindTypes .= 'd'; 
        } else if (is_string($value)) { 
          $sBindTypes .= 's'; 
        } else {
          $sBindTypes .= 'b';
        }

        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($value, 'Added Value from $aData to $aTableValues2use[]'); }
      } // if ( in_array($key, $aTableColumns) ) {
    } // foreach
    $sQuestionMarks = rtrim($sQuestionMarks, ', ');
 
    // If $aTableColumns2use or $aTableValues2use are empty, that means no values matched
    if ( (sizeof($aTableColumns2use) == 0) || (sizeof($aTableValues2use) == 0) ) {
        // Set $Error if no values matched
        //$this->Error = "Error: No values were passed that matched any columns.";
        return FALSE;
    } else {
        // Implode $aTableColumns2use into a prepared SQL insert statement
        $query = "INSERT INTO ".$tbl." (".implode(", ",$aTableColumns2use). ") VALUES (".$sQuestionMarks. ")";

        /* create a prepared statement */
        $stmt =  $this->stmt_init();
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($query, 'MySQL Wrapper Object: $this->prepare(...)'); }
        if ( ! $stmt->prepare($query) ) {
          throw new Exception("MySQL prepare: QUERY=prepare($query), ERROR=" . $this->error);
        }

        // Insert $sBindTypes as 1st elt of $aTableValues2use
        array_unshift( $aTableValues2use, $sBindTypes );

        // $stmt->bind_param(...
        /* Previous to PHP 5.3, this worked...
        if ( call_user_func_array( array($stmt, 'bind_param'), $aTableValues2use ) === FALSE) {
          throw new Exception("MySQL bind_param ($aTableValues2use): " . $this->error);
        }
        */

        // $stmt->bind_param(...
        // Post PHP 5.3, we need to do use $this->makeValuesReferenced()...
        if ( call_user_func_array( array($stmt, 'bind_param'), $this->makeValuesReferenced($aTableValues2use) ) === FALSE) {
          throw new Exception("MySQL bind_param ($aTableValues2use): " . $this->error);
        }

        if (! $stmt->execute() ) {
          /* Figure out why this failed - maybe a unique field value (eg, "user") is already taken?
           * It could be more reliable/portable to issue a SELECT query here.  We would
           * definitely need to do that (or at least include code to do it) if we were
           * supporting multiple kinds of database backends, not just MySQL.  However,
           * the prepared statements interface we're using is MySQL-specific anyway. */
          if ($this->errno === 1062 /* ER_DUP_ENTRY */)
            throw new Exception('This entry already exists in the DB and duplicates are not allowed for this field:<br />' . $this->error);
          else
            throw new Exception("MySQL execute: QUERY=$query, ERROR=" . $this->error);
        }

        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($stmt, 'Query completed without error. Requesting the number of rows affected for $stmt:'); }
        $numRowsAdded = $stmt->affected_rows;

        $stmt->close();

        return $numRowsAdded;
    }
  } // addArray2DB()

  public function updateDBfromArray($aData, $tbl, $idName, $id) {
  // Method that dynamically updates values in a MYSQL DB table using the $aData vars
  // DOES: UPDATE ... WHERE $idName=$id (to identify which row to update)
  // USES: prepared statment protocol for queries that are efficient and SQL-injection safe
  // RETURNS: # of rows added or FALSE if no table cols matched the input array
  //
  // NOTES: See "NOTES" section of addArray2db()
  //
    if ($id <0) throw new Exception(__METHOD__." (A negative index was received: $id)");

    $aTableColumns = array(); // column names as pulled from db table
    $aTableColumns2use = array(); // column names that match $aData form fields
    $aTableValues2use = array(); // parallel array to tableColumns2use, with corresponding values
    $sBindTypes = '';
 
    // Pull the column names from the table $tbl
    $query = "SHOW COLUMNS FROM ".$tbl;

    if ( ! $pullColumns = $this->runQuery($query) ) {
      throw new Exception("MySQL pull columns: QUERY=$query, ERROR=" . $this->error);
    }

    // Pull an associative array of column names and put them into a non-associative array
    while ( $columns = mysqli_fetch_assoc($pullColumns) ) {
        $aTableColumns[] = $columns["Field"]; // grabs "Field"name field of $pullColumns ary
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($columns['Field'], 'Adding Column from DB to $aTableColumns[]'); }
    }
    if (DEBUG_FIREPHP) { global $firephp; $firephp->info($aTableColumns, '$aTableColumns'); }

    // Check to see if the variables match up with the column names
    // If so, store in $aTableColumns2use
    foreach( $aData as $key => $value ) {
      if ( $value==DEFAULT_NONREQUIRED_FIELD_VALUE ) { $value = DEFAULT_TABLE_INDEX; }
      if (DEBUG_FIREPHP) { global $firephp; $firephp->info($key, '$key'); }
      if (DEBUG_FIREPHP) { global $firephp; $firephp->info($value, '$value'); }

      if ( empty($value) ) {
        if (! DB_ALLOW_EMPTY_VALUES) { // empty values not allowed
          continue; // then skip this field entirely
        } else if (DB_WRITE_EMPTY_VALUES_AS_DEFAULT) {
          // if get here, we know empty values are allowed/DB_ALLOW_EMPTY_VALUES==TRUE
          $value = 'DEFAULT';
        } else if (DB_WRITE_EMPTY_VALUES_AS_NULL) {
          // if get here, we know empty values are allowed/DB_ALLOW_EMPTY_VALUES==TRUE
          $value = NULL;
        }
      }

      if ( in_array($key, $aTableColumns) ) {
        // Put the column name into the array
        $aTableColumns2use[] = $key;
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($key, 'Added Column from $aData to $aTableColumns2use[]'); }

        // NOTE: May need to add something like this: $aTableValues2use = addcslashes($value, '%_');
        //       if expect % or _ in data, and will parse later with ...WHERE color LIKE '%red%'
        if (stripos($value, 'DEFAULT') === FALSE) {
          $aTableValues2use[] = $value;
          if ( isset($sSet) ) {
            $sSet .= ", $key=?";
          } else {
            $sSet = "$key=?";
          }

          // Determine type of each $value for use in the subsequent $stmt->bind_param() call
          if (is_int($value)) {
            $sBindTypes .= 'i'; 
          } else if (is_double($value)) {
            $sBindTypes .= 'd'; 
          } else if (is_string($value)) {
            $sBindTypes .= 's'; 
          } else {
            $sBindTypes .= 'b';
          }
        } else {
          if ( isset($sSet) ) {
            $sSet .= ", $key=DEFAULT";
          } else {
            $sSet = "$key=DEFAULT";
          }
        }

        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($value, 'Added Value from $aData to $aTableValues2use[]'); }
      } // if ( in_array($key, $aTableColumns) ) {
    } // foreach
 
    // If $aTableColumns2use or $aTableValues2use are empty, that means no values matched
    if ( (sizeof($aTableColumns2use) == 0) || (sizeof($aTableValues2use) == 0) ) {
        // Set $Error if no values matched
        //$this->Error = "Error: No values were passed that matched any columns.";
        return FALSE;
    } else {
        // Implode $aTableColumns2use into a prepared SQL insert statement
        //$query = "UPDATE $tbl SET " .implode('=?, ', $aTableColumns2use). "=? WHERE $idName=?";
        // New Way: Use $sSet to allow "SET column=DEFAULT..."
        $query = "UPDATE $tbl SET $sSet WHERE $idName=?";

        /* create a prepared statement */
        $stmt =  $this->stmt_init();
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($query, 'MySQL Wrapper Object: $this->prepare(...)'); }
        if ( ! $stmt->prepare($query) ) { 
          throw new Exception("MySQL prepare: QUERY=prepare($query), ERROR=" . $this->error);
        }

        // Append $id as last elt of $aTableValues2use
        // Determine type of each $id for use in the subsequent $stmt->bind_param() call
        if (is_int($id)) {
          $sBindTypes .= 'i'; 
        } else if (is_double($id)) { 
          $sBindTypes .= 'd'; 
        } else {
          $sBindTypes .= 's'; 
        }
        $aTableValues2use[] = $id;
        // Insert $sBindTypes as 1st elt of $aTableValues2use
        array_unshift( $aTableValues2use, $sBindTypes );
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($aTableValues2use, 'MySQL Wrapper Object: $aTableValues2use'); }

		$aTmp = array();
		$aTmp = $this->makeValuesReferenced($aTableValues2use);
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($aTmp, '$aTmp:'); }

        // $stmt->bind_param(...
		// Prior to PHP 5.3, this was fine:
		// if ( call_user_func_array( array($stmt, 'bind_param'), $aTableValues2use ) === FALSE) {
		// Now, tho, we need to send an array of references (not values) to call_user_func_array()
        if ( call_user_func_array( array($stmt, 'bind_param'), $this->makeValuesReferenced($aTableValues2use) ) === FALSE) {
          throw new Exception("MySQL bind_param ($aTableValues2use): " . $this->error);
        }

        if (! $stmt->execute() ) {
          /* Figure out why this failed - maybe a unique field value (eg, "user") is already taken?
           * It could be more reliable/portable to issue a SELECT query here.  We would
           * definitely need to do that (or at least include code to do it) if we were
           * supporting multiple kinds of database backends, not just MySQL.  However,
           * the prepared statements interface we're using is MySQL-specific anyway. */
          if ($this->errno === 1062 /* ER_DUP_ENTRY */)
            throw new Exception('This entry already exists in the DB and duplicates are not allowed for this field:<br />' . $this->error);
          else
            throw new Exception("MySQL execute: QUERY=$query, ERROR=" . $this->error);
        }

        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($stmt, 'Query completed without error. Requesting the number of rows affected for $stmt:'); }
        $numRowsUpdated = $stmt->affected_rows;

        $stmt->close();

        return $numRowsUpdated;
    }
  } // updateDBfromArray()

    /* ******* HELPER METHODS ********* */

  private function serializeArrayValues($aData) {
  // Handle case where form sends an array (eg, checkboxes):  <input type="text" name="FacilNameAry[]" />
  // Convert this array to a string suitable for storing in a DB, or a summed total if SUM_CHECKBOXES
    if (SUM_CHECKBOXES) { // add 'em up, numerically
        $total = 0;
        foreach ($aData as $key => $value) {
            $total += $value;
            if (DEBUG_FIREPHP) { global $firephp; $firephp->info($total, "\$total array summation"); }
        }
        return $total;
    } else { // non-numeric group, serialize (concatenate) using DB_DELIM
        $sSerialized = '';
        foreach ($aData as $key => $value) {
            // First, remove any empty values, etc.
            $value = trim($value, " \t\n\r\x0B");
            if ( $value==DEFAULT_NONREQUIRED_FIELD_VALUE ) { $value = DEFAULT_TABLE_INDEX; }

            if ( ! empty($value) ) {
                $sSerialized .= DB_DELIM . $value;
            }
        }
        if (! empty($sSerialized)) {
            return $sSerialized . DB_DELIM;
        } else {
            return $sSerialized;
        }
        // retrieve via explode(), or $query="SELECT * FROM <tableName> WHERE <fieldName> LIKE '%*red*%'";
        // OR, search for multiple matches via:
        //     $lookingForColors = array('red', 'green', 'yellow');
        //     $whereStr = "<fieldName> LIKE '%*".implode("*%' OR LIKE '%*",$lookingForColors).'*%'";
        //     $query = "SELECT * FROM <tableName> WHERE $whereStr";
    } // if  (SUM_CHECKBOXES) {
  }

  public function sanitizeRequestVars($aREQUEST, $bSerializeArrays=TRUE) {
  // $aREQUEST is typically $_POST, $_REQUEST, etc.
  // RETURNS: new, sanitized array, leaving orig array untouched
    $aReturnArray = array();

    foreach ($aREQUEST as $name => $value) {
        if ( $bSerializeArrays && isset($value) && is_array($value) ) {
            $value = $this->serializeArrayValues($value);
        } else {
            $value = trim($value, " \t\n\r\x0B");
        }

        if ( $value==DEFAULT_NONREQUIRED_FIELD_VALUE ) { $value = DEFAULT_TABLE_INDEX; }
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($name, '$name'); }
        if (DEBUG_FIREPHP) { global $firephp; $firephp->info($value, '$value'); }

        // Verify chosen fields
        switch ($name) {
        case 'user' :
          /* Sanity-check the username, don't rely on our use of prepared statements
           * alone to prevent attacks on the SQL server via malicious usernames. */
          // Regular username
          #if (!preg_match('/^[a-zA-Z0-9_]{1,60}$/', $user))
          // Email username
          //if (! filter_var($value, FILTER_VALIDATE_EMAIL) ) // BAD: allows a@b.c
          // ** Prev (less accurate than below)
          //if (!preg_match('/^[a-zA-Z0-9][\.a-zA-Z0-9_-]*@([a-zA-Z0-9][a-zA-Z0-9_-]+\.)+[a-zA-Z]{2,4}$/', $value))
          
          //$value = filter_var($value, FILTER_SANITIZE_EMAIL);
          if (!preg_match('/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]{2,}(\.[a-z0-9-]{2,})+$/i', $value)) {
            throw new Exception('Invalid email address given for login id');
          }
          break;
        case 'pass' :
          if (strlen($value) > MAX_PSWD_LENGTH)
            throw new Exception('The supplied password is too long');
          break;
        case 'id' :
          if (!preg_match('/^[0-9]{1,10}$/', $value)) {
            if (DEBUG_FIREPHP) $firephp->log('Exiting, hacker input detected for id');
            throw new Exception('Invalid id');
          }
          break;
        default:
        } // switch()

/* Magic Quotes is a deprecated feature of PHP. When enabled, which it is on many
 web servers, the PHP interpreter will automagically escape many inputs to PHP
 scripts. This may be desirable to provide some minimal security for poorly-written
 PHP scripts that fail to defend themselves against SQL injections, but with
 properly-written PHP scripts this feature may actually be more of a problem.

Specifically, unless we deal with this issue, a password containing the single quote
character might or might not reach our PHP application with the character
escaped. If the magic_quotes_gpc PHP setting is then toggled, or if our PHP
application install is moved to another system where this setting is set
differently, the password will stop working.

Thus, we need to check whether magic_quotes_gpc was set and undo its effect at least
for specific inputs where this matters. */
        if ( get_magic_quotes_gpc() ) {
          $aReturnArray[$name] = stripslashes($value);
        } else {
          $aReturnArray[$name] = $value;
        }
    } // foreach ($aRequest

    return $aReturnArray;
  } // sanitizeRequestVars()

}
?>
