<?php
/*********************************************************
 * @version    SVN: $Id$
 * @author     Rich Raffals
 ***********************************************************
 */
require('Form.php');

class WordSearchForm extends Form {
  public $searchWord = '';
  public $aUrlList = array();

  ///////////////////////////////////////////////////////////////////
  public function __construct( $queryParam=NULL ) {
  ///////////////////////////////////////////////////////////////////
    parent::__construct($queryParam);
  }

  //
  // Methods
  //
  ///////////////////////////////////////////////////////////////////
  public function sanitize( $sParamName='Search Word' ) {
  ///////////////////////////////////////////////////////////////////
  // Set $this->searchWord to a safe/validated/sanitized version of $sParamName and return it
  // Calls $this->reportError() if can't produce safe search word from $sParamName
    if ( ! empty($this->sRawQueryParam) ) {
      $safeSearchWord = parent::sanitize($sParamName); // generic form field sanitization

      if ( strlen($safeSearchWord) < 4 ) {
        $this->reportError('"'.$safeSearchWord.'" is too short -- only words > 3 characters are stored');
      }

      if ( ! preg_match('/\w/', $safeSearchWord) ) {
        $this->reportError('Invalid characters in search word');
      }

      $this->bStatus = TRUE;
      return $this->searchWord = $safeSearchWord;      
    } // if (! empty($this->sRawQueryParam))

    $this->reportError('No search word entered');
  } // sanitize()


  ///////////////////////////////////////////////////////////////////
  public function lookupUrlsContainingWord() {
  ///////////////////////////////////////////////////////////////////
  // Looks up $this->searchWord and count of times it appears for each URL in DB_WORD_TABLE
  // RETURNS: $aUrlList[url, wordCount] and saves as $this->aUrlList
  // Calls $this->reportError() on application failure; Throws exception on DB errors
    if ( ! empty($this->searchWord) ) {
      require(PHP_REQUIRE_ROOT . 'Model/MySQLiWrapper.php');

      try { // connect to MySQL
        $db = MySQLiWrapper::getInstance( array(DB_HOST, DB_USER, DB_PSWD, DB_NAME, DB_PORT) );
      } catch(Exception $e) {
        $this->reportError("Can't access DB: " . $e->getMessage());
      }

      // create/execute a prepared statement to get word count for urls
      $query = 'SELECT count, url FROM ' .DB_WORD_TABLE. ' WHERE word=? ORDER BY count DESC';
      $bindParam = $this->searchWord;  $sBindType = 's';
      try {
        $stmt =  $db->executePrepared($query, $sBindType, $bindParam);
      } catch(Exception $e) {
        $this->reportError('DB action failed: ' . $e->getMessage());
      }

      // put results in array (NOTE: key: $url, value: $count)
      $stmt->bind_result($count, $url);
      $aUrlList = array();
      while ( $stmt->fetch() ) {
        $aUrlList[$url] = $count;
      }
      $stmt->close();

      if ( count($aUrlList) ) {
        $this->bStatus = TRUE;
        return $this->aUrlList = $aUrlList;
      } else {
        $this->reportError('"'.$this->searchWord.'" does not appear in any of the pages in the DB');
      }

    } // If (! empty($this->searchWord))

    $this->reportError('Search word is empty');
  } // countWordsOnPage()

}
?>
